<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV101-56-g411a2eed:2023-07-24#

namespace Platiti\GPWebPayGpeNs\PrestaShop;

use Platiti\GPWebPayGpeNs\UniModul\ConfigFieldType;
use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\Http;
use Platiti\GPWebPayGpeNs\UniModul\IDatabaseConnection;
use Platiti\GPWebPayGpeNs\UniModul\OrderStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItem;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItemType;
use Platiti\GPWebPayGpeNs\UniModul\UniCustomerStatistics;
use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;
use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\PrePayGWInfo;

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use Validate;
use Product;
use ReflectionProperty;
use SmartyLazyRegister;
use State;
use Tax;
use Tools;
use Address;
use Cache;
use Cart;
use Configuration;
use Country;
use Currency;
use Customer;
use DateTime;
use Db;
use Error;
use Exception;
use HelperForm;
use Language;
use Order;
use OrderHistory;
use OrderState;
use PaymentModule;
use AdminController;
use Shop;


use function Platiti\GPWebPayGpeNs\UniModul\BeginUniErr;
use function Platiti\GPWebPayGpeNs\UniModul\EndUniErr;
use function Platiti\GPWebPayGpeNs\UniModul\ResetUniErr;
use function Platiti\GPWebPayGpeNs\UniModul\ExceptionUniErr;
use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\create_initialize_object;


if (!defined('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath')) define('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath', _PS_ROOT_DIR_ . '/UniModul');

require_once(__DIR__."/lib/UniModul/UniModul.php");

class PrestaAdapter extends PaymentModule
{
	/** @var string */
	protected $_html = '';

	/** @var UniModul */
	public $uniModul;
	/** @var PaymentModule */
	protected $paymentModule;
	/** @var ConfigInfo */
	protected $configInfo;
	/** @var bool */
	protected $alwaysCreateOrderBeforeProcess;

	/**
	 * @param ?PaymentModule $paymentModule
	 * @param ?string $uniModul
	 * @param ?string $subMethod
	 * @return void
	 */
	public function __construct($paymentModule=null, $uniModul=null, $subMethod = null)
	{
		try {
			BeginUniErr();

			if (is_null($paymentModule)) {      // pro pripad, ze je Adapter inicializovany jako Prestovy modul sam o sobe, tak procedeme nektere zakladni inicializace
				$paymentModule = $this;
			}
			$this->paymentModule = $paymentModule;

		//var_dump($paymentModule);
			if (_PS_VERSION_ >= '1.4.0') {
				$paymentModule->tab = 'payments_gateways';
			} else {
				$paymentModule->tab = 'Payment';
			}
			$paymentModule->version = preg_replace("/\D*0*(\d+)-(\d+)-.*/", "$1.$2", "#Ver:PRV101-56-g411a2eed:2023-07-24#");
	//		$this->currencies = true;
	//		$this->currencies_mode = 'radio';


			if (is_null($uniModul)) {
				$paymentModule->name = 'PrestaAdapter';  // musi byt shodne s Binder modulem, Presta toto pole pouziva
				$paymentModule->displayName = "PrestaAdapter - servis modul";
				parent::__construct();
				EndUniErr();
				return;
			}

			$paymentModule->name = $uniModul."Binder".$subMethod;  // musi byt shodne s Binder modulem, Presta toto pole pouziva

			//$this->description = $this->l('Accepting credit and debit card payments with');
			$paymentModule->confirmUninstall = $this->l('Are you sure you want to delete your details ?');

			$this->paymentModule->bootstrap = true;
			parent::__construct();

			$uniFact = new UniModulFactory();

			$this->configInfo = $uniFact->getConfigInfo($uniModul, $this->getLanguage(), $subMethod);

			$configSetting = $this->getConfigData($this->configInfo, $uniModul);
			$this->uniModul = $uniFact->createUniModul($uniModul, $configSetting, $subMethod);
			$paymentModule->displayName = $this->uniModul->getModuleSubMethodName($this->getLanguage(), '') . " (".$paymentModule->name.")";

			if ($this->uniModul->name != 'GPWebPayGpe') {
				$paymentModule->author = "www.platiti.cz";
			} else {
				$paymentModule->author = "GPE";
			}

			$this->alwaysCreateOrderBeforeProcess = $this->uniModul->name == 'PlatimPak' || $this->uniModul->name == 'Twisto2';

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param string $name
	 * @param ?string $uniModulName
	 * @return string
	 */
	function getShortConfigName($name, $uniModulName=null) {
		if ($uniModulName===null) {  // protoze pri konstrukci jeste uniModul neni dosazen
			$uniModulName = $this->uniModul->name;
		}
		$nn = $uniModulName . "B" . "_" . $name;
		$nn = substr($nn, 0, 32);  // max omezeni na delku cfg itemu
		return $nn;
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param string $uniModulName
	 * @return ConfigSetting
	 */
	protected function getConfigData($configInfo, $uniModulName) {
		$configData = array();
		foreach ($configInfo->configFields as $configField) {
			if ($configField->type == ConfigFieldType::$orderStatusMultiple) {
				$configData[$configField->name] = explode(',', (string)Configuration::get($this->getShortConfigName($configField->name, $uniModulName)));
			} else {
				$configData[$configField->name] = Configuration::get($this->getShortConfigName($configField->name, $uniModulName));
			}
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->databaseConnection = new PrestaDbConnection();
		if ("GPWebPayGpeNs" == "Pti"."UniNs") { // @phpstan-ignore-line
			$uniModulConfig->uniModulDirUrl = $this->getHttpHost().__PS_BASE_URI__."/modules/PrestaAdapter/lib/UniModul/";
		} else {
			$uniModulConfig->uniModulDirUrl = $this->getHttpHost().__PS_BASE_URI__."/modules/".$uniModulName."Binder/lib/UniModul/";
		}
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');
		$uniModulConfig->adapterName = 'PrestaShop';
		$uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');
		$uniModulConfig->legacyUniModulVarPaths = [_PS_ROOT_DIR_ . '/modules/PrestaAdapter/lib/UniModul'];
		//_DB_PREFIX_ ..
		//if (_DB_TYPE_ != 'MySQL') user_error('Eshop pouziva jinou nez mysql databazi');  // 1.5 uz zas pouziva jen MySql
		$cfgs = create_initialize_object(ConfigSetting::class, array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}

	public function l($string, $specific = false, $id_lang = null) {
		try {
			BeginUniErr();
			$tmp = $this->name;
			if ("GPWebPayGpeNs" == "Pti"."UniNs") { // @phpstan-ignore-line
				$this->name = "PrestaAdapter";
			} else {
				$this->name = $this->paymentModule->name;
			}
			if ($specific === false) $specific = 'PrestaAdapterClass';
			$ret = parent::l($string, $specific, $id_lang); // Note: PS 1.6 function l(,) has only 2 params
			$this->name = $tmp;
			return EndUniErr($ret);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	////// instalace

	/**
	 * @return bool
	 */
	public function install()
	{
		try {
			BeginUniErr();
			$failinst = !call_user_func(array($this->paymentModule, 'PaymentModule::install')); /** @phpstan-ignore-line */
			if ($failinst) {
				$this->uniModul->logger->writeLog('WARN ADAPTER: PaymentModule::install failed');
			}
			$failinst = $failinst || !$this->upgrade();
			return EndUniErr(!$failinst);

		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
 * @return bool
 */
	public function upgrade()
	{
		try {
			BeginUniErr();
			if ($this->uniModul != null) {
				$this->uniModul->logger->writeLog("INFO ADAPTER: Upgrading");
				$failinst =
				   !$this->paymentModule->registerHook('payment')
				OR !$this->paymentModule->registerHook('paymentOptions')
				OR !$this->paymentModule->registerHook('paymentReturn')
				OR !$this->paymentModule->registerHook('adminOrder')
				OR !$this->paymentModule->registerHook('extraRight')
				OR !$this->paymentModule->registerHook('actionPDFInvoiceRender')
				//OR !$this->paymentModule->registerHook('actionOrderStatusUpdate')
				OR !$this->paymentModule->registerHook('actionOrderStatusPostUpdate')
				//OR !$this->paymentModule->registerHook('actionOrderHistoryAddAfter')
				OR !$this->paymentModule->registerHook('displayProductAdditionalInfo');
				// if ($this->uniModul->subMethod == null) {
				// 	$failinst = $failinst
				// 	OR !$this->registerHook('leftColumn');
				// }
				if ($failinst) {
					$this->uniModul->logger->writeLog('WARN ADAPTER: Failed registering hooks');
				}

				$this->uniModul->moduleInstallVarDir();
				$this->uniModul->moduleInstallDatabase();

				if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
					$this->uniModul->logger->writeLog("INFO ADAPTER: Installing legacy endpoints");
					$destDir = _PS_MODULE_DIR_ . '/PrestaAdapter';
					if (!file_exists($destDir)) mkdir($destDir);
					copy(__DIR__ . "/lib/legacy/notification.php", $destDir . "/notification.php");
					copy(__DIR__ . "/lib/legacy/validation.php", $destDir . "/validation.php");
					copy(__DIR__ . "/lib/legacy/callback.php", $destDir . "/callback.php");
					//chmod($destDir . "/notification.php", 0444);
				}

				return EndUniErr(!$failinst);
			} else {
				return EndUniErr(true);
			}
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param mixed $params
	 * @return ?string
	 */
	public function hookDisplayProductAdditionalInfo($params)	{
		// hook existuje az v PS 1.7, zatimco hookExtraRight jen do PS 1.6
		return $this->hookExtraRight($params);
	}



	/**
	 * @param mixed $params
	 * @return ?string
	 */
	public function hookExtraRight($params)	{

		if (!$this->paymentModule->active)
			return null;

		BeginUniErr();

		global $smarty;

		$id_product = Tools::getValue('id_product');

		if (is_numeric($id_product)) {

			$current_currency = isset($this->context->currency->iso_code) ? $this->context->currency->iso_code : $GLOBALS['currency']->iso_code;

			$product = new Product((int)($id_product));
			$product_price = $product->getPrice(Product::$_taxCalculationMethod == PS_TAX_INC);

			$shopBaseUrl = $this->getHttpHost().__PS_BASE_URI__;
			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml($shopBaseUrl, $current_currency, $product_price);
			$embedHtml = '<div style="text-align:center;margin:1em" class="UniInstallment">' . $embedHtml . '</div>';
			return EndUniErr($embedHtml);
		}
		return EndUniErr(null);

	}




	////// zobrazeni v administraci a konfigurace

	/**
	 * @return string
	 */
	public function getContent()
	{
		try {
			BeginUniErr();
			if ($this->uniModul != null) {
				$this->_html = '<h2>'.$this->uniModul->getModuleSubMethodName($this->getLanguage(), '') . " (".$this->paymentModule->name.")".'</h2>';
				if (isset($_POST['submitUniAdapter']) || isset($_POST['btnSubmit']))
				{
					if ($this->uniModul->subMethod == null) {
						foreach ($this->configInfo->configFields as $configField) {
							if ($configField->type == ConfigFieldType::$subMethodsSelection) {
								$submehods = $configField->choiceItems + $this->uniModul->getSubMethods();
								$cumar = array();
								foreach ($submehods as $sname) {
									if (isset($_POST[$this->getShortConfigName($configField->name).'_'.$sname])) {
										$cumar[]=$sname;
									}
								}
								Configuration::updateValue($this->getShortConfigName($configField->name), strval(implode(' ', $cumar)));
							} else if ($configField->type == ConfigFieldType::$orderStatus && in_array($configField->name, array('orderStatusSuccessfull', 'orderStatusPending', 'orderStatusFailed'))) {
								// preskakujeme
							} else if ($configField->type == ConfigFieldType::$orderStatusMultiple) {
								Configuration::updateValue($this->getShortConfigName($configField->name), implode(',', isset($_POST[$this->getShortConfigName($configField->name)]) ? $_POST[$this->getShortConfigName($configField->name)] : array()));
							} else {
								Configuration::updateValue($this->getShortConfigName($configField->name), strval($_POST[$this->getShortConfigName($configField->name)]));
							}
						}

						$fldnameNormal = 'orderStatusSuccessfull'; Configuration::updateValue($this->getShortConfigName($fldnameNormal), strval($_POST[$this->getShortConfigName($fldnameNormal)]));
						$fldnameNormal = 'orderStatusPending'; Configuration::updateValue($this->getShortConfigName($fldnameNormal), strval($_POST[$this->getShortConfigName($fldnameNormal)]));
						$fldnameNormal = 'orderStatusFailed'; Configuration::updateValue($this->getShortConfigName($fldnameNormal), strval($_POST[$this->getShortConfigName($fldnameNormal)]));
						if (!$this->alwaysCreateOrderBeforeProcess) {
							$fldnameNormal = 'createOrderBeforePayment'; Configuration::updateValue($this->getShortConfigName($fldnameNormal), isset($_POST[$this->getShortConfigName($fldnameNormal)]) || isset($_POST[$this->getShortConfigName($fldnameNormal).'_']));
						}
						$fldnameNormal = 'showConfirmationPage'; Configuration::updateValue($this->getShortConfigName($fldnameNormal), strval($_POST[$this->getShortConfigName($fldnameNormal)]));

					}
					$this->displayConf();
				}

				if (_PS_VERSION_ >= '1.6.0' ) {
					$this->displayFormSettings();
				} else {
					$this->displayFormSettings15();
				}
			} else {
				$this->_html = '<h2>UniAdapter - servis modul</h2>  no settings';
			}
			return EndUniErr($this->_html);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}



	/**
	 * @return void
	 */
	public function displayConf()
	{
		try {
			BeginUniErr();
			if (_PS_VERSION_ >= '1.6.0' ) {
				$this->_html .= $this->displayConfirmation($this->l('Settings updated'));
			} else {
				$this->_html .= '
				<div class="conf confirm">
					<img src="../img/admin/ok.gif" alt="'.$this->l('Confirmation').'" />
					'.$this->l('Settings updated').'
				</div>';
			}
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @return void
	 */
	protected function displayFormSettings()
	{
		$input = array();
		$values = array();

		if ($this->uniModul->subMethod == null) {
			foreach ($this->configInfo->configFields as $configField) {
				$fldname = $this->getShortConfigName($configField->name);
				$conf = Configuration::getMultiple(array($fldname));
				$value = array_key_exists($fldname, $_POST) ? $_POST[$fldname] : (array_key_exists($fldname, $conf) ? $conf[$fldname] : '');

				if ($configField->type == ConfigFieldType::$text) {
					$input[] = array('type' => 'text', 'label' => $configField->label, 'name' => $fldname);
					$values[$fldname] = $value;
				} else if ($configField->type == ConfigFieldType::$choice) {
					$options = array();
					foreach ($configField->choiceItems as $val=>$lab) {
						$options[] = array('val' => $val, 'lab' => $lab);
					}
					$input[] = array('type' => 'select', 'label' => $configField->label, 'name' => $fldname,
						'options' => array('query' => $options, 'id' => 'val', 'name' => 'lab'));
					$values[$fldname] = $value;
				} else if ($configField->type == ConfigFieldType::$subMethodsSelection) {
					$submehods = $configField->choiceItems + $this->uniModul->getSubMethods();
					$language = $this->getLanguage();
					$cumar = explode(' ', $value);
					$options = array();
					$checks = array();
					foreach ($submehods as $sname) {
						$subname = $this->uniModul->getModuleSubMethodName($language, $sname);
						$options[] = array('val' => $sname, 'lab' => $subname);
						$checked = isset($_POST['btnSubmit']) ? isset($_POST[$fldname.'_'.$sname]) : (in_array($sname, $cumar));
						if ($checked) {
							$checks[] = $fldname.'_'.$sname;
							$values[$fldname.'_'.$sname] = 1;
						}

					}
					$input[] = array('type' => 'checkbox', 'label' => $configField->label, 'name' => $fldname,
						'values' => array('query' => $options, 'id' => 'val', 'name' => 'lab'));
					$values[$fldname] = implode(',', $checks);
				} else if ($configField->type == ConfigFieldType::$orderStatusMultiple || $configField->type == ConfigFieldType::$orderStatus && !in_array($configField->name, array('orderStatusSuccessfull', 'orderStatusPending', 'orderStatusFailed'))) {
					list($inp, $val) = $this->displayFormSettings_orderStatus($configField->name, $configField->type == ConfigFieldType::$orderStatusMultiple, $configField->type == ConfigFieldType::$orderStatusMultiple);
					$input[] = $inp; $values += $val;
				}
			}
		}
		// orderStatusy
		list($inp, $val) = $this->displayFormSettings_orderStatus('orderStatusSuccessfull');
		$input[] = $inp; $values += $val;
		list($inp, $val) = $this->displayFormSettings_orderStatus('orderStatusPending');
		$input[] = $inp; $values += $val;
		list($inp, $val) = $this->displayFormSettings_orderStatus('orderStatusFailed');
		$input[] = $inp; $values += $val;

		if (!$this->alwaysCreateOrderBeforeProcess) {
			// createOrderBeforePayment
			$fldnameNormal = 'createOrderBeforePayment';
			$label = $this->l('Create order before payment');
			$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
			$input[] = array('type' => 'checkbox', 'label' => $label, 'name' => $fldname,
							'values' => array('query' => array(array('val' => '', 'lab' => '')), 'id' => 'val', 'name' => 'lab'));
			$checked = isset($_POST[$fldname.'_']) ? isset($_POST[$fldname.'_']) : Configuration::get($fldname);
			$values[$fldname.'_'] = $checked;
		}
		// showConfirmationPage
		$fldnameNormal = 'showConfirmationPage';
		$label = $this->l('Show confirmation page');
		$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
		$value = isset($_POST[$fldname]) ? $_POST[$fldname] : Configuration::get($fldname);
		$confirmationPageOptions = array(0 => $this->l('Always'), 1 => $this->l('If message'), 2 => $this->l('Never'));
		$options = array();
		foreach ($confirmationPageOptions as $val=>$lab) {
			$options[] = array('val' => $val, 'lab' => $lab);
		}
		$input[] = array('type' => 'select', 'label' => $label, 'name' => $fldname,
			'options' => array('query' => $options, 'id' => 'val', 'name' => 'lab'));
		$values[$fldname] = $value;

		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => $input,
				'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? : 0;
		$helper->id = (int)Tools::getValue('id_carrier');
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'btnSubmit';
		$helper->currentIndex = $this->paymentModule->context->link->getAdminLink('AdminModules', false).'&configure='
			.$this->paymentModule->name.'&tab_module='.$this->paymentModule->tab.'&module_name='.$this->paymentModule->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		assert(is_a($this->context->controller, AdminController::class));
		$helper->tpl_vars = array(
			'fields_value' => $values,
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		$form = $helper->generateForm(array($fields_form));
		$this->_html .= $form;
	}

	/**
	 * @param string $fldnameNormal
	 * @param bool $nullable
	 * @param bool $multiple
	 * @return array{array<string, mixed>, array<string, mixed>}
	 */
	private function displayFormSettings_orderStatus($fldnameNormal, $nullable = false, $multiple = false) {
		$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
		$label = $this->uniModul->dictionary->get($fldnameNormal, $this->getLanguage());
		$cookie = isset($this->context->cookie) ? $this->context->cookie : $GLOBALS['cookie']; // phpStan: $this->context->cookie may be null in older PS versions
		$orderStates = OrderState::getOrderStates($cookie->id_lang); // phpStan: PS1.7: Cookie::$id_lang is set but not defined
		$value = isset($_POST[$fldname]) ? $_POST[$fldname] : Configuration::get($fldname);
		$options = array();
		if ($nullable) {
			$options[] = array('val' => null, 'lab' => $this->uniModul->dictionary->get('nullOrderStatus', $this->getLanguage()));
		}
		foreach ($orderStates as $orderState) {
			$val = $orderState['id_order_state'];
			$lab = $orderState['name'];
			$options[] = array('val' => $val, 'lab' => $lab);
		}
		$input = array('type' => 'select', 'label' => $label, 'name' => $fldname . ($multiple ? '[]' : ''),
			'options' => array('query' => $options, 'id' => 'val', 'name' => 'lab'));
		if ($multiple) {
			$input['multiple'] = true;
			$value = isset($_POST[$fldname]) ? Http::postMulti($fldname) : explode(',', (string)Configuration::get($fldname));
			$fldname .= '[]';
		}
		$values = array($fldname => $value);
		return array($input, $values);
	}


	/**
	 * @return void
	 */
	protected function displayFormSettings15()
	{
		try {
			BeginUniErr();
			$this->_html .= '
			<form action="'.$_SERVER['REQUEST_URI'].'" method="post" style="clear: both;">


			<fieldset>
				<legend><img src="../img/admin/contact.gif" />'.$this->l('Settings').'</legend>

				';

			if ($this->uniModul->subMethod == null) {
				foreach ($this->configInfo->configFields as $configField) {
					if ($configField->type != ConfigFieldType::$orderStatus || !in_array($configField->name, array('orderStatusSuccessfull', 'orderStatusPending', 'orderStatusFailed'))) {
						$fldname = $this->getShortConfigName($configField->name);
						$conf = Configuration::getMultiple(array($fldname));
						$value = array_key_exists($fldname, $_POST) ? $_POST[$fldname] : (array_key_exists($fldname, $conf) ? $conf[$fldname] : '');

						$this->_html .= '<label>'.$configField->label.'</label>';
						if ($configField->type == ConfigFieldType::$text) {
							$this->_html .= '<div class="margin-form"><input type="text" size="33" name="'.$fldname.'" value="'.$value.'" /></div><br/>';
						} else if ($configField->type == ConfigFieldType::$choice) {
							$this->_html .= '<td><select name="'.$fldname.'">';
							foreach ($configField->choiceItems as $val=>$lab) {
								$this->_html .= '<option value="'.$val.'" '.(($val !== null && $val == $value)?' selected="selected"':'').'> '.$lab. '</option>\n';
							}
							$this->_html .= '</select></td>';
						} else if ($configField->type == ConfigFieldType::$subMethodsSelection) {
							$this->_html .= '<td>';

							$submehods = $configField->choiceItems + $this->uniModul->getSubMethods();

							$language = $this->getLanguage();
							$cumar = explode(' ', $value);
							foreach ($submehods as $sname) {
								$subname = $this->uniModul->getModuleSubMethodName($language, $sname);
								$checked = isset($_POST[$fldname.'_submit']) ? isset($_POST[$fldname.'_'.$sname]) : (in_array($sname, $cumar));
								$this->_html .= '<div class="margin-form"><input type="checkbox" name="'.$fldname.'_'.$sname.'" value="'.$sname.'" '.($checked?' checked ':'').' /> '.$subname.'</div><br/>';
							}
							$this->_html .= '<input type="hidden" name="'.$fldname.'_submit" value="x" />';
							$this->_html .= '</td>';
						} else if ($configField->type == ConfigFieldType::$orderStatusMultiple || $configField->type == ConfigFieldType::$orderStatus) {
							$this->_html .= $this->displayFormSettings_orderStatus15($configField->name, $configField->type == ConfigFieldType::$orderStatusMultiple, $configField->type == ConfigFieldType::$orderStatusMultiple);
						}

						$this->_html .= '<div style="clear: both">&nbsp;</div>';  // je to proto, kdyz dlouhy nazev, tak se rozhodi layout
					}
				}
			}

			// orderStatusy
			$this->_html .= '<label>'.$this->uniModul->dictionary->get('orderStatusSuccessfull', $this->getLanguage()).'</label>';
			$this->_html .= $this->displayFormSettings_orderStatus15('orderStatusSuccessfull');
			$this->_html .= '<label>'.$this->uniModul->dictionary->get('orderStatusPending', $this->getLanguage()).'</label>';
			$this->_html .= $this->displayFormSettings_orderStatus15('orderStatusPending');
			$this->_html .= '<label>'.$this->uniModul->dictionary->get('orderStatusFailed', $this->getLanguage()).'</label>';
			$this->_html .= $this->displayFormSettings_orderStatus15('orderStatusFailed');

			if (!$this->alwaysCreateOrderBeforeProcess) {
				// createOrderBeforePayment
				$fldnameNormal = 'createOrderBeforePayment';
				$label = $this->l('Create order before payment');
				$this->_html .= '<label>'.$label.'</label>';
				$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
				$checked = isset($_POST[$fldname]) ? isset($_POST[$fldname]) : Configuration::get($fldname);
				$this->_html .= '<div class="margin-form"><input type="checkbox" name="'.$fldname.'" value="1" '.($checked?' checked ':'').' /></div><br/>';
				$this->_html .= '<input type="hidden" name="'.$fldname.'_submit" value="x" />';
			}

			// showConfirmationPage
			$fldnameNormal = 'showConfirmationPage';
			$label = $this->l('Show confirmation page');
			$this->_html .= '<label>'.$label.'</label>';
			$this->_html .= $this->uniModul->getMiniLogoSpan();
			$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
			$this->_html .= '<td><select name="'.$fldname.'">';
			$value = isset($_POST[$fldname]) ? $_POST[$fldname] : Configuration::get($fldname);
			$confirmationPageOptions = array(0 => $this->l('Always'), 1 => $this->l('If message'), 2 => $this->l('Never'));
			foreach ($confirmationPageOptions as $confiCode => $confiName) {
				$this->_html .= '<option value="'.$confiCode.'" '.(($confiCode !== null && $confiCode == $value)?' selected="selected"':'').'> '.$confiName. '</option>\n';
			}
			$this->_html .= '</select></td>';
			$this->_html .= '<div style="clear: both">&nbsp;</div>';  // je to proto, kdyz dlouhy nazev, tak se rozhodi layout

			///

			$this->_html .= '
				<br /><center><input type="submit" name="submitUniAdapter" value="'.$this->l('Update settings').'" class="button" /></center>
				</fieldset>
				</form><br /><br />
				';
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param string $fldnameNormal
	 * @param bool $nullable
	 * @param bool $multiple
	 * @return string
	 */
	private function displayFormSettings_orderStatus15($fldnameNormal, $nullable = false, $multiple = false) {
		global $cookie;
		$fldname = $this->getShortConfigName($fldnameNormal, $this->uniModul->name);
		$orderStates = OrderState::getOrderStates($GLOBALS['cookie']->id_lang);
		$html = '<td><select name="'.$fldname. ($multiple ? '[]' : '') . '"' . ($multiple ? ' multiple' : '') . '>';
		$value = isset($_POST[$fldname]) ? $_POST[$fldname] : Configuration::get($fldname);
		if ($nullable) {
			$val = null;
			$lab = $this->uniModul->dictionary->get('nullOrderStatus', $this->getLanguage());
			$orderStates = array(array('id_order_state' => $val, 'name' => $lab)) + $orderStates;
		}
		foreach ($orderStates as $orderState) {
			$val = $orderState['id_order_state'];
			$lab = $orderState['name'];
			if ($multiple) {
				$value = isset($_POST[$fldname]) ? Http::postMulti($fldname) : explode(',', (string)Configuration::get($fldname));
			}
			$selected = ($val !== null && $val == $value) || (is_array($value) && in_array($val, $value));
			$html .= '<option value="'.$val.'" '.($selected?' selected="selected"':'').'> '.$lab. "</option>\n";
		}
		$html .= '</select></td>';
		$html .= '<div style="clear: both">&nbsp;</div>';  // je to proto, kdyz dlouhy nazev, tak se rozhodi layout
		return $html;
	}

	///////////// placeni

	/**
	 * @param mixed $params
	 * @return ?string
	 */
	public function hookPayment($params)
	{
		try {
			BeginUniErr();
			global $smarty;

			if (!$this->paymentModule->active)
				return EndUniErr(null);
			if (!$this->_checkCurrency($params['cart']))
				return EndUniErr(null);

			$language = $this->getLanguage();

			//$orderToPayInfo = new OrderToPayInfo();
			//$orderToPayInfo->language = $language;
			$orderToPayInfo = $this->getOrderToPayInfo();


			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
			if (!$prePayGWInfo->isPossible) {
				return EndUniErr();
			}

			if ($prePayGWInfo->subMethods==null) {
				$prePayGWInfo->subMethods=array('');
			}
			$submethods = $prePayGWInfo->subMethods;

			// $GLOBALS['UniPrestaShop_MethodsOrder'] = array('metoda1', 'metoda2'), nejmenovane metody se zaradi na konec v puvodnim poradi, nenalezene metody jsou ignorovany (uzitecne pokud instalovano vice modulu), obecna metodda je ''
			if (isset($GLOBALS['UniPrestaShop_MethodsOrder'])) {
				$newOrder = $GLOBALS['UniPrestaShop_MethodsOrder'];
				$submethods = array_intersect($submethods, $newOrder) + array_diff($submethods, $newOrder);
			}

			$submetTempl = [];
			$tpl15 = '';
			foreach ($submethods as $sname) {
				$submetTempl[$sname]['uniModulLogo'] = $this->uniModul->getModulSubMethodLogoImage2($sname);
				$submetTempl[$sname]['methodName'] = $this->uniModul->getModuleSubMethodName($language, $sname);

				if (_PS_VERSION_ >= '1.5.0') {
					$submetTempl[$sname]['redirUrl'] = $this->context->link->getModuleLink(strval($this->paymentModule->name),'Redirect', array('subMethod' => $sname));
					$tpl15 = '.15';
				} else {
					$submetTempl[$sname]['redirUrl'] = $this->getHttpHost().__PS_BASE_URI__."modules/PrestaAdapter/redirect.php?uniModul=".urlencode($this->uniModul->name).'&subMethod='.urlencode($sname);
				}
				// (toto v PS 1.6 nefunguje, zjistit proc) $submetTempl[$sname]['redirUrl'] = $this->context->link->getModuleLink($this->paymentModule->name,'Redirect', array('subMethod' => $sname));
			}
			$smarty->assign('submetTempl', $submetTempl);
			if ("GPWebPayGpeNs" == "Pti"."UniNs") { // @phpstan-ignore-line
				$tmodule = 'PrestaAdapter';
			} else {
				$tmodule = $this->paymentModule->name;
			}
			return EndUniErr($this->display($tmodule, 'PrestaAdapter'.$tpl15.'.tpl'));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param mixed $params
	 * @return ?array<int, PaymentOption>
	 */
	public function hookPaymentOptions($params) {
		try {
			BeginUniErr();

			if (!$this->paymentModule->active)
				return EndUniErr(null);
			if (!$this->_checkCurrency($params['cart']))
				return EndUniErr(null);

			$language = $this->getLanguage();
			$orderToPayInfo = $this->getOrderToPayInfo();
			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
			if (!$prePayGWInfo->isPossible) {
				return EndUniErr(null);
			}

			if ($prePayGWInfo->subMethods==null) {
				$prePayGWInfo->subMethods=array('');
			}
			$submethods = $prePayGWInfo->subMethods;

			// $GLOBALS['UniPrestaShop_MethodsOrder'] = array('metoda1', 'metoda2'), nejmenovane metody se zaradi na konec v puvodnim poradi, nenalezene metody jsou ignorovany (uzitecne pokud instalovano vice modulu), obecna metodda je ''
			if (isset($GLOBALS['UniPrestaShop_MethodsOrder'])) {
				$newOrder = $GLOBALS['UniPrestaShop_MethodsOrder'];
				$submethods = array_intersect($submethods, $newOrder) + array_diff($submethods, $newOrder);
			}

			$paymentOptions = array();
			foreach ($submethods as $sname) {
				$externalOption = new PaymentOption();
				$externalOption->setCallToActionText($this->uniModul->getModuleSubMethodName($language, $sname))
					->setAction($this->context->link->getModuleLink(strval($this->paymentModule->name), 'Redirect', array('subMethod' => $sname)))
					//->setAdditionalInformation($this->context->smarty->fetch('module:paymentexample/views/templates/front/payment_infos.tpl'))
					->setLogo($this->uniModul->getModulSubMethodLogoImage2($sname));
				$paymentOptions[] = $externalOption;
			}
			return EndUniErr($paymentOptions);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param mixed $params
	 * @return ?string
	 */
	public function hookPaymentReturn($params)
	{
		try {
			BeginUniErr();
			if (!$this->paymentModule->active)
				return EndUniErr(null);

			global $smarty;
			$smarty->assign('binderName', $this->name);

			$methodName = $this->uniModul->getModuleSubMethodName($this->getLanguage(), isset($_GET['submethod']) ? $_GET['submethod'] : '' );
			$smarty->assign('methodName', $methodName);
			if (isset($this->context)) {  /** @phpstan-ignore-line */ //pro PS<=1.4 je to shop_name jiz nastaveno z jadra
				$smarty->assign('shop_name', $this->context->shop->name);
			}
			EndUniErr();
			BeginUniErr(E_UNIERR_DEFAULT & ~E_WARNING);
			return EndUniErr($this->display(__DIR__.'/PrestaAdapter.php', 'confirmation.tpl'));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param mixed $params
	 * @return void
	 */
	public function hookActionPDFInvoiceRender($params) {
		if (_PS_VERSION_ >= '1.7.0.0' && _PS_VERSION_ < '1.7.0.3') {
			$rp = new ReflectionProperty('SmartyLazyRegister', 'registry');
			$rp->setAccessible(true);
			$a = $rp->getValue(SmartyLazyRegister::getInstance($this->smarty));
			unset($a['displayPriceSmarty']);
			$rp->setValue(SmartyLazyRegister::getInstance($this->smarty), $a);
		}
	}

	/**
	 * @param mixed $params
	 * @return void
	 */
	public function hookActionOrderStatusUpdate($params) {
		/*
		BeginUniErr();
		//$this->uniModul->logger->writeLog("hookActionOrderStatusUpdate");
		$id_order = $params['id_order'];
		$order = new Order($id_order);
		if ($order->module == $this->uniModul->name . "Binder") {
			$this->uniModul->shopOrderStatusChanged(null, $order->id_cart, $params['newOrderStatus']->id);  // nepouzivam shopOrderNumber, ale jen cartId, protoze shopOrderNumber se muze menit globalem mezi reference a order_id
		}
		EndUniErr();
		*/
	}

	/**
	 * @param mixed $params
	 * @return void
	 */
	public function hookActionOrderStatusPostUpdate($params) {
		BeginUniErr();
		//$this->uniModul->logger->writeLog("hookActionOrderStatusPostUpdate");
		$id_order = $params['id_order'];
		$order = new Order($id_order);
		if ($order->module == $this->uniModul->name . "Binder") {
			$this->uniModul->shopOrderStatusChanged($id_order, (string)$order->id_cart, (string)$order->current_state);
		}
		EndUniErr();
	}

	/**
	 * @param mixed $params
	 * @return void
	 */
	public function hookActionOrderHistoryAddAfter($params) {
		/*
		BeginUniErr();
		//$this->uniModul->logger->writeLog("hookActionOrderHistoryAddAfter");
		$id_order = $params['order_history']->id_order;
		$order = new Order($id_order);
		if ($order->module == $this->uniModul->name . "Binder") {
			$this->uniModul->shopOrderStatusChanged(null, $order->id_cart, $order->current_state);  // nepouzivam shopOrderNumber, ale jen cartId, protoze shopOrderNumber se muze menit globalem mezi reference a order_id
		}
		EndUniErr();
		*/
	}

	/**
	 * @param int $id_cart
	 * @param int $id_order_state
	 * @param float $amountPaid
	 * @param string $paymentMethod Payment method (eg. 'Credit card')
	 * @param string|null $message Message to attach to order
	 * @param mixed $extraVars
	 * @param int|null $currency_special
	 * @param bool $dont_touch_amount
	 * @param string|bool $secure_key
	 * @param ?Shop $shop
	 * @param string|null $order_reference
	 */
	public function validateOrder($id_cart, $id_order_state, $amountPaid, $paymentMethod = 'Unknown', $message = NULL, $extraVars = array(), $currency_special = NULL, $dont_touch_amount = false, $secure_key = false, Shop $shop = null, $order_reference = null)
	{
		try {
			BeginUniErr();
			if (!$this->paymentModule->active)
				return EndUniErr(false);

			BeginUniErr(E_UNIERR_DEFAULT & ~E_WARNING & ~E_USER_DEPRECATED);
			$r = call_user_func(array($this->paymentModule, 'PaymentModule::validateOrder'), $id_cart, $id_order_state, $amountPaid, $paymentMethod, $message, $extraVars, $currency_special, $dont_touch_amount, $secure_key, $shop, $order_reference);  /** @phpstan-ignore-line */
			EndUniErr();
			EndUniErr();
			return $r;
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
				$this->procesReplyStatus($orderReplyStatus, false);
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param bool $frontend_redir
	 * @return void
	 */
	public function procesReplyStatus($orderReplyStatus, $frontend_redir=false) {
		global $cookie;

		switch ($orderReplyStatus->orderStatus)
		{
			case OrderStatus::$successful:
				$newState = Configuration::get($this->getShortConfigName('orderStatusSuccessfull', $this->uniModul->name));
				$guiOk = true;
				break;
			case OrderStatus::$pending:
				$newState = Configuration::get($this->getShortConfigName('orderStatusPending', $this->uniModul->name));
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
			case OrderStatus::$invalidReply;
				$newState = Configuration::get($this->getShortConfigName('orderStatusFailed', $this->uniModul->name));
				$guiOk = false;
				break;
			default:
				throw new Exception("Invalid orderReplyStatus " . $orderReplyStatus->orderStatus);
		}


		if ($orderReplyStatus->orderStatus != OrderStatus::$invalidReply || $frontend_redir) {  // invalid reply resime jen pro frontend, jinak ignorujeme

			BeginSynchronized();
			$cart = new Cart(intval($orderReplyStatus->shopPairingInfo));
			if (_PS_VERSION_ >= '1.5.0') {
				Cache::clean('Cart::orderExists_'.(int)$cart->id); // od verze 1.5.6.1 minimalne to jinak zdvojuje ordery
			}
			$orderExists = $cart->OrderExists();	//?? if (Validate::isLoadedObject($cart) AND $cart->OrderExists() == false)
			if (!$orderExists) {
				if ($orderReplyStatus->orderStatus == OrderStatus::$successful || $orderReplyStatus->orderStatus == OrderStatus::$pending) {   // objednavku vytvorime jen pro pozitivni vysledky

					$this->paymentModule->context->currency = new Currency((int) $cart->id_currency);
					$this->paymentModule->context->customer = new Customer((int) $cart->id_customer);
					$order_total = $orderReplyStatus->uniAdapterData['order_total'];
					$currency_order = new Currency(intval($cart->id_currency));
					BeginUniErr(E_UNIERR_DEFAULT & ~E_WARNING & ~E_USER_DEPRECATED);
					$cart_order_total = $cart->getOrderTotal(true, 3);
					EndUniErr();
					$order_correct = $cart_order_total
						&& (!isset($orderReplyStatus->uniAdapterData['currency_iso_code']) || $orderReplyStatus->uniAdapterData['currency_iso_code'] == $currency_order->iso_code);

					if (!$order_correct) {
						$this->uniModul->logger->writeLog("ADAPTER: Incorrect cart vs payment amount. Setting PS_OS_ERROR status. Payment: " . $order_total . " " . (isset($orderReplyStatus->uniAdapterData['currency_iso_code']) ? $orderReplyStatus->uniAdapterData['currency_iso_code'] : "XXX") . ", Cart: " . $cart->getOrderTotal(true, 3) . " " . $currency_order->iso_code);
						$newState = Configuration::get('PS_OS_ERROR'); // toto je jen fix pro PS 1.7.6.0-1, kde nefunguje detekce spatne castky a kde se to nenastavuje samo, dulezita je i ta mena
					}

					$uniAdapterData = $orderReplyStatus->uniAdapterData;
					$uniAdapterData['order_total_ps'] = $cart_order_total; // castka objednavky v okamziku vytvoreni
					$uniAdapterData['currency_iso_code_ps'] = $currency_order->iso_code; // mena objednavky v okamziku vytvoreni
					$transactionRecord = $this->uniModul->getOrderTransactionRecordFromDbLast(null, (string)$cart->id);
					if ($transactionRecord === null) throw new Exception('Missing transactionRecord');
					$this->uniModul->updateUniAdapterDataInDb($transactionRecord->transactionPK, $uniAdapterData);
					$extraVars = array('transaction_id' => $orderReplyStatus->gwOrderNumber);
					$paymentName = $this->uniModul->getModuleSubMethodName($this->getLanguage(), $orderReplyStatus->uniAdapterData['subMethod']);
					if ($newState === false) throw new Error("Newstate is not configured");
					$this->validateOrder($cart->id, (int)$newState, $order_total, $paymentName, NULL, $extraVars, $cart->id_currency, false, $cart->secure_key);
					$order = new Order($this->paymentModule->currentOrder);
					$this->uniModul->updateShopOrderNumber($orderReplyStatus, (string)$this->getShopOrderNumber($order));
					$this->uniModul->logger->writeLog("ADAPTER: Creating shop order number ".$this->getShopOrderNumber($order).", cart:".$cart->id.", new order state ".$newState);
				} else {
					$order = null;
				}
			} else {   // order uz existuje, jen ho updatnu
				$orderId = Order::getOrderByCartId((int)$orderReplyStatus->shopPairingInfo);
				$order = new Order($orderId);

				$transactionRecord = $this->uniModul->getOrderTransactionRecordFromDbLast(null, (string)$order->id_cart);
				if ($transactionRecord === null) throw new Exception('Missing transactionRecord');
				$order_correct =
					!isset($transactionRecord->uniAdapterData['order_total_ps']) // zpetna kompatibilita
					|| ($transactionRecord->uniAdapterData['order_total'] == $transactionRecord->uniAdapterData['order_total_ps']
					&& $transactionRecord->uniAdapterData['currency_iso_code'] == $transactionRecord->uniAdapterData['currency_iso_code_ps']);
				if ($order_correct) {
					//if ($orderReplyStatus->shopOrderNumber === null or $orderReplyStatus->shopOrderNumber=='') {
					//	user_error('Chybi shopOrderNumber, mozna refresh na odeslanou platbu...');
					//}
					$transientStates = array(
						Configuration::get($this->getShortConfigName('orderStatusPending', $this->uniModul->name)),
						Configuration::get($this->getShortConfigName('orderStatusFailed', $this->uniModul->name)),
						);
					// stavy PS_OS_OUTOFSTOCK_PAID nebo starsi PS_OS_OUTOFSTOCK jsou jako ze zaplaceno, takze uz je menit nebudeme
					// PS_OS_OUTOFSTOCK_UNPAID menit budeme krome pripadu pendingu, o kterem to v podstate je
					if ($orderReplyStatus->orderStatus != OrderStatus::$pending && Configuration::get('PS_OS_OUTOFSTOCK_UNPAID') !== false) {
						$transientStates[] = Configuration::get('PS_OS_OUTOFSTOCK_UNPAID');
					}
					if ($newState === false) throw new Error("Newstate is not configured");
					if ($order->getCurrentState() != $newState && in_array($order->getCurrentState(), $transientStates)) {
						$history = new OrderHistory();
						$history->id_order = (int)$order->id;
						BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_WARNING & ~E_WARNING & ~E_USER_DEPRECATED);
						$history->changeIdOrderState((int)$newState, $order->id);
						$history->addWithemail(true  /*, $extraVars*/);
						EndUniErr();
						$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number ".$this->getShopOrderNumber($order).", cart:".$cart->id.", old order state ".$order->getCurrentState().", new order state ".$newState);
					} else {
						$this->uniModul->logger->writeLog("ADAPTER: Not updating shop order number ".$this->getShopOrderNumber($order).", cart:".$cart->id.", old order state ".$order->getCurrentState().", new order state ".$newState);
					}
				} else {
					$this->uniModul->logger->writeLog("ADAPTER: Incorrect cart at order cration vs payment amount. Won't touch order status. Payment: " . $orderReplyStatus->uniAdapterData['order_total'] . " " . (isset($orderReplyStatus->uniAdapterData['currency_iso_code']) ? $orderReplyStatus->uniAdapterData['currency_iso_code'] : "XXX") . ", Cart at order creation: " . $transactionRecord->uniAdapterData['order_total_ps'] . " " . $transactionRecord->uniAdapterData['currency_iso_code_ps']);
				}
			}
			EndSynchronized();
		} else {
			$order = null;
		}

		if ($frontend_redir) {
			if ($guiOk) {
				assert($order != null);
				if (_PS_VERSION_ >= '1.5.0') {
					$rurl = $this->paymentModule->context->link->getPageLink('order-confirmation') . '?id_cart='.$order->id_cart.'&id_module='.$this->id.'&id_order='.$order->id.'&key='.$order->secure_key.'&submethod='.$orderReplyStatus->uniAdapterData['subMethod'];
				} else {
					$rurl = __PS_BASE_URI__.'order-confirmation.php?id_cart='.$order->id_cart.'&id_module='.$this->id.'&id_order='.$order->id.'&key='.$order->secure_key;
				}
				$uer = ResetUniErr();
				$this->uniModul->jsRedirect($rurl);
				ResetUniErr($uer); /** @phpstan-ignore-line */
			} else {
				if ($order != null) {
					// musime obnovit kosik
					$oldCart = new Cart(Order::getCartIdStatic((int)$order->id, (int)$cookie->id_customer));
					BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
					$duplication = $oldCart->duplicate();
					EndUniErr();
					assert($duplication !== true);
					if (!$duplication OR !Validate::isLoadedObject($duplication['cart'])) {
						// nelze obnovit, bude prazdny ..., pak pridat logovani toho
						//$this->errors[] = Tools::displayError('Sorry, we cannot renew your order.');
						//elseif (!$duplication['success'])
						//$this->errors[] = Tools::displayError('Missing items - we are unable to renew your order');
					} else {
						$cookie->id_cart = $duplication['cart']->id;
						$cookie->write();
					}
				}

				if (_PS_VERSION_ >= '1.5.0') {
					$failUrl = $this->context->link->getModuleLink(strval($this->paymentModule->name),'Fail', array('errorMessage' => $orderReplyStatus->resultText.''));
				} else {
					$failUrl = $this->getHttpHost().__PS_BASE_URI__."modules/PrestaAdapter/fail.php?uniModul=".urlencode($this->uniModul->name)."&errorMessage=".urlencode($orderReplyStatus->resultText);  // TODO urlencode
				}
				$uer = ResetUniErr();
				$this->uniModul->jsRedirect($failUrl);
				ResetUniErr($uer); /** @phpstan-ignore-line */
			}
		}
	}



	//// zobrazeni v administraci v detailu objednavky

	/**
	 * @param mixed $params
	 * @return string
	 */
	public function hookAdminOrder($params)	{
		try {
			BeginUniErr();
			if ($this->uniModul->subMethod != null) {
				return EndUniErr(''); // pro submoduly (jen PayUCreditCard) nehookujem, zajisti to ten hlavni
			}
			$order_id = (int)$params['id_order'];
			$order = new Order($order_id);
			$transactionRecord = $this->uniModul->getOrderTransactionRecordFromDbLast(null, (string)$order->id_cart);
			if ($transactionRecord === null) {
				return EndUniErr(''); // hook se vola pro pro vsechnyh moduly, i kdyz nejsou pro tuto objednavku urceny
			}

			$language = $this->getLanguage();
			$order_correct =
				!isset($transactionRecord->uniAdapterData['order_total_ps']) // zpetna kompatibilita
				|| ($transactionRecord->uniAdapterData['order_total'] == $transactionRecord->uniAdapterData['order_total_ps']
				&& $transactionRecord->uniAdapterData['currency_iso_code'] == $transactionRecord->uniAdapterData['currency_iso_code_ps']);

			if (!$order_correct) {
				$statusTexts = array(OrderStatus::$successful => 'successful', OrderStatus::$pending => 'pending', OrderStatus::$failedFinal => 'failed', OrderStatus::$failedRetriable => 'failed');
				$amountMismatchMsg = "Incorrect cart vs payment amount. Payment: " . $transactionRecord->uniAdapterData['order_total'] . " " . $transactionRecord->uniAdapterData['currency_iso_code']
				. ", Cart: "  . $transactionRecord->uniAdapterData['order_total_ps'] . " " . $transactionRecord->uniAdapterData['currency_iso_code_ps']
				. ". Payment result: " . $statusTexts[$transactionRecord->orderStatus];
			}

			$innerHtml = $this->paymentModule->displayName;
			$innerHtml .= "<p/>" . $this->uniModul->dictionary->get('orderDetailGwOrdNum', $language). ': '.$transactionRecord->gwOrderNumber;
			if ($transactionRecord->forexNote !== null) {
					$innerHtml .= "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $language). ': ' . $transactionRecord->forexNote;
			}
			if (isset($amountMismatchMsg)) $innerHtml .= "<p/><span style='color:red;'>" . $amountMismatchMsg . "</span>";
			$innerHtml .= "<p/>" . $this->uniModul->getOrderAdminDetailHtml($transactionRecord->shopOrderNumber, $transactionRecord->shopPairingInfo);


			if (_PS_VERSION_ >= '1.7.7') {
				$this->_html .= '
					<div class="product-row row">
						<div class="col-md-12 left-column">
							<div class="card mt-2 d-print-none">
								<div class="card-header">
									<div class="row">
										<div class="col-md-6">
										<i class="material-icons">money</i>
										'.$this->l('Payment module info').'
										</div>
									</div>
								</div>
								<div class="card-body">
								' . $innerHtml . '
								</div>
							</div>
						</div>
					</div>
				';
			} else if (_PS_VERSION_ >= '1.6.0') {
				$this->_html .= '
					<div class="row">
						<div class="col-lg-12">
							<div class="panel">
								<div class="panel-heading">
									<i class="icon-money"></i>
									'.$this->l('Payment module info').'
								</div>
								' . $innerHtml . '
							</div>
						</div>
					</div>
				';
			} else {
				$this->_html .= '
				<br />
				<fieldset style="width:400px">
					<legend><img src="../img/admin/gold.gif" /> '.$this->l('Payment module info') /* to se vypisuje vzdy EN ale melo by byt ok!? */.'</legend>
					' . $innerHtml . '
				</fieldset>';
			}

			return EndUniErr($this->_html);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param ?string $subMethod
	 * @param bool $possiblyCreateOrder
	 * @return OrderToPayInfo
	 */
	public function getOrderToPayInfo($subMethod=null, $possiblyCreateOrder=false) {
		global $cookie;
		$cart = new Cart(intval($cookie->id_cart));
		$state = NULL;
		$customer = new Customer(intval($cart->id_customer));
		$currency_order = new Currency(intval($cart->id_currency));
		//VEN $currency_module = $this->getCurrency();

		BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
		$order_total = $cart->getOrderTotal(true, 3);
		EndUniErr();

		// volitelne zaokrouhleni tak, jak to presta zobrazuje, (i kdyz to ma vnitrne presta nezaoukrouhene) a pak se musi pri potvrzeni platby stejne udat nezaokrouhlena castka, jinak presta varuje
		$c_decimals = (int)$currency_order->decimals * _PS_PRICE_DISPLAY_PRECISION_;
		if (isset($GLOBALS['UniPrestaShop_RoundPrice']) && $GLOBALS['UniPrestaShop_RoundPrice']) {
			$order_total_round = Tools::ps_round($order_total, $c_decimals);
		} else {
			$order_total_round = $order_total;
		}

		/** @var array<int, array{iso_code: string, conversion_rate: float}> */
		$currencies = Currency::getCurrencies(false, false);
		$currencyRates = array();
		foreach ($currencies as $cur) {
			$currencyRates[$cur['iso_code']] = $cur['conversion_rate'];
		}

		if ($possiblyCreateOrder && ($this->alwaysCreateOrderBeforeProcess || Configuration::get($this->getShortConfigName('createOrderBeforePayment', $this->uniModul->name)))) {
			$newState = Configuration::get($this->getShortConfigName('orderStatusPending', $this->uniModul->name));
			$mailVars = array();
			$paymentName = $this->uniModul->getModuleSubMethodName($this->getLanguage(), $subMethod);
			if ($newState === false) throw new Error("Newstate is not configured");
			$this->validateOrder($cart->id, (int)$newState, $order_total, $paymentName, NULL, $mailVars, $cart->id_currency, false, $cart->secure_key);
			$order = new Order($this->paymentModule->currentOrder);
			$orderId = $this->getShopOrderNumber($order);
			$this->uniModul->logger->writeLog("ADAPTER: created order before payment ".$this->getShopOrderNumber($order).", ref:".$orderId.", cart:".$cart->id.", new order state ".$newState);
		} else {
			$orderId = null;
		}


		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $subMethod;
		$orderToPayInfo->shopOrderNumber = (string)$orderId;
		$orderToPayInfo->shopPairingInfo = (string)$cart->id;
		$orderToPayInfo->amount = $order_total_round;
		$orderToPayInfo->currency = $currency_order->iso_code;
		$orderToPayInfo->language = $this->getLanguage();
		$orderToPayInfo->description = (string)Configuration::get('PS_SHOP_NAME');

		$address = new Address((int)($cart->id_address_delivery));
		$country = new Country((int)($address->id_country));
		if ($address->id_state)	{
			$state	= new State((int)$address->id_state);
			$state_iso_code = $state->iso_code;
		} else {
			$state_iso_code = null;
		}

		$customerData = new CustomerData();
		$customerData->first_name = $customer->firstname;
		$customerData->last_name = $customer->lastname;
		$customerData->email = $customer->email;
		$customerData->street = $address->address1.' '.$address->address2;
		$customerData->city = $address->city;
		$customerData->post_code = $address->postcode;
		$customerData->state = $state_iso_code;
		$customerData->country = $country->iso_code;
		$customerData->phone = !trim((string)$address->phone) ? $address->phone_mobile : $address->phone;
		$customerData->identifier = (string)$customer->id;  // pro guetsa presta vzdy vytvori noveho uzivatele pro kazdou objednavku
		$orderToPayInfo->customerData = $customerData;

		BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
		$sumdet = $cart->getSummaryDetails();
		EndUniErr();
		$cartItems = array();

		$products = isset($sumdet['gift_products']) ? array_merge($sumdet['products'], $sumdet['gift_products']) : $sumdet['products'];

		foreach ($products as $product) {
			$cartItem = new UniCartItem();
			$cartItem->productId = $product['id_product'];
			$cartItem->name = $product['name'];
			$cartItem->quantity = $product['cart_quantity'];
			$cartItem->unitPrice = $product['price_wt'];
			$cartItem->unitTaxAmount = $product['price_wt'] - $product['total']/$product['cart_quantity'];
			if (abs($cartItem->unitTaxAmount)<0.01) { $cartItem->unitTaxAmount = 0; }   // navic Presta ma nekdy 'total' zaokrouhlene a 'price_wt' ne, takze vznika i zaporna dan
			$cartItem->taxRate = $cartItem->unitTaxAmount == 0 ? 0 : $product['rate'];  //pri pouziti EuropeanVAT se dan nuluje, ne ovsem jeji procenta, jinak bych je samzrejem mohl i vypocitat, ale pak zase nastavaji problemy s presnosti
			$cartItem->type = UniCartItemType::commodity;
			$cartItems[] = $cartItem;

		}

		$cartItem = new UniCartItem();
		$cartItem->name = 'shipping / doprava';
		$cartItem->quantity = 1;
		BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
		$cartItem->unitPrice = $cart->getOrderTotal(true, Cart::ONLY_SHIPPING) + $cart->getOrderTotal(true, Cart::ONLY_WRAPPING);
		$bezDane = $cart->getOrderTotal(false, Cart::ONLY_SHIPPING) + $cart->getOrderTotal(false, Cart::ONLY_WRAPPING);
		EndUniErr();
		$cartItem->unitTaxAmount = $cartItem->unitPrice - $bezDane;
		if (method_exists($sumdet['carrier'], 'getTaxesRate')) {
			assert(is_object($sumdet['carrier']));
			$cartItem->taxRate = $sumdet['carrier']->getTaxesRate($sumdet['invoice']);
		} else {   // pro PS 1.4
			$cartItem->taxRate = (float)Tax::getCarrierTaxRate($cart->id_carrier, (int)$cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')});
		}
		$cartItem->type = UniCartItemType::delivery;
		$cartItems[] = $cartItem;
		$cartItemShipping = $cartItem;

		// vouchery
		foreach ($sumdet['discounts'] as $disc) {
			if ($disc['free_shipping']) {   // u cart rule s free shipping ale bez kodu slevy, se nenaplnuje hodnota slevy, takze obratime shipping
				$cartItem = new UniCartItem();
				$cartItem->name = $disc['description'];
				$cartItem->quantity = 1;
				$cartItem->unitPrice = -$cartItemShipping->unitPrice;
				$cartItem->unitTaxAmount = -$cartItemShipping->unitTaxAmount;
				$cartItem->taxRate = $cartItemShipping->taxRate;
				$cartItem->type = UniCartItemType::discount;
				$cartItems[] = $cartItem;
			}
			if ($disc['value_real'] != 0 && $disc['value_real'] != $cartItemShipping->unitPrice) {   // u cart rule s free shipping ale bez kodu slevy, se nenaplnuje hodnota slevy, takze neresime
				$cartItem = new UniCartItem();
				$cartItem->name = $disc['description'];
				$cartItem->type = UniCartItemType::discount;
				$cartItem->quantity = 1;
				$cartItem->unitPrice = -$disc['value_real'];
				$cartItem->unitTaxAmount = -($disc['value_real'] - $disc['value_tax_exc']);
				if ($disc['free_shipping']) {  // odecteme cisla za shipping
					$cartItem->unitPrice += $cartItemShipping->unitPrice;
					$cartItem->unitTaxAmount += $cartItemShipping->unitTaxAmount;
				}
				$cartItem->taxRate = round($cartItem->unitTaxAmount / ($cartItem->unitPrice - $cartItem->unitTaxAmount) * 100);
				if (!in_array($cartItem->taxRate, array(21,15,10))) { // pokud je dan nejaka smichana, rozpocteme do nejvyssi sazby a zbytek do nulove
					// secteni pres sazby
					/** @var array<int, float> */
					$rozdeleni = array_reduce($cartItems, function($a, $ci) {
						$a[(int)$ci->taxRate] = $ci->unitPrice * $ci->quantity + (isset($a[(int)$ci->taxRate]) ? $a[(int)$ci->taxRate] :0);
						return $a;
					}, array());

					// co se vejde dame do nejvyssi sazby
					$sazby = array_keys($rozdeleni);
					rsort($sazby);
					$sazbamax = $sazby[0];
					if ($sazbamax != 0) {
						$castka = $cartItem->unitTaxAmount * (100 + $sazbamax) / $sazbamax;
						$zbytek = $cartItem->unitPrice - $castka;
						$cartItem->unitPrice = $castka;
						$cartItem->taxRate = $sazbamax;
						$cartItems[] = $cartItem;
					} else {
						$zbytek = $cartItem->unitPrice;
					}
					// zbytek dame do nulove sazby
					$cartItem = new UniCartItem();
					$cartItem->name = $disc['description'];
					$cartItem->quantity = 1;
					$cartItem->unitPrice = $zbytek;
					$cartItem->unitTaxAmount = 0;
					$cartItem->taxRate = 0;
				}
				$cartItems[] = $cartItem;
			}
		}

		$orderToPayInfo->cartItems = $cartItems;

		//statistiky
		if (_PS_VERSION_ >= '1.7.' && !$customer->isGuest() && $customer->date_add !== null) {
			$stats = new UniCustomerStatistics();
			$stats->registeredDate = new DateTime($customer->date_add);
			/** @var Currency|false */
			$cur = Currency::getDefaultCurrency();
			if ($cur !== false) {
				$stats->currency = $cur->iso_code;

				$orders = Order::getCustomerOrders($cart->id_customer, true);
				$total = 0;
				$maximal = 0;
				$count = 0;
				$lastOrderDate = 0;
				foreach ($orders as $o) {
					if ($o['valid']) {
						$total += $o['total_paid'];
						$maximal = max($maximal, (float)$o['total_paid']);
						$lastOrderDate = max($lastOrderDate, $o['date_add']);
						$count++;
					}
				}
				if ($lastOrderDate !== 0) $stats->lastOrderDate = new DateTime($lastOrderDate);
				$stats->orderCount = $count;
				$stats->orderTotalSum = $total;
				$stats->orderTotalMax = $maximal;

				$orderToPayInfo->customerStatistics = $stats;
			} else {
				$this->uniModul->logger->writeLog("No default currency for statistics");
			}
		}



		$replyUrl = $this->context->link->getModuleLink(strval($this->paymentModule->name),'Reply');
		$orderToPayInfo->replyUrl = $replyUrl;

		$notifyUrl = $this->context->link->getModuleLink(strval($this->paymentModule->name),'Notify');
		$orderToPayInfo->notifyUrl = $notifyUrl;

		$orderToPayInfo->uniModulDirUrl = $this->uniModul->baseConfig->uniModulDirUrl;

		$orderToPayInfo->currencyRates = $currencyRates;

		$orderToPayInfo->uniAdapterData = array('subMethod' => $orderToPayInfo->subMethod, 'order_total'=>$order_total, 'currency_iso_code'=>$currency_order->iso_code);

		return $orderToPayInfo;
	}

	/**
	 * @param Order $order
	 * @return string
	 */
	protected function getShopOrderNumber($order) {
		if (isset($order->reference) && empty($GLOBALS['UniPrestaShop_UseOrderId'])) {  /** @phpstan-ignore-line */ //reference neni pro presta 1.4
			return $order->reference;
		} else {
			return (string)$order->id;
		}
	}


	/**
	 * @return void
	 */
	function redirectToGatewayPage() {
		$orderToPayInfo = $this->getOrderToPayInfo($_GET['subMethod']);
		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);

		$showConfirmationPage = Configuration::get($this->getShortConfigName('showConfirmationPage', $this->uniModul->name));

		if (!(isset($_REQUEST['uniPayCont']) || $showConfirmationPage == 2 || ($showConfirmationPage == 1 && $prePayGWInfo->forexMessage == "" && !$prePayGWInfo->selectCsPayBrand) )) {
			$this->assignSmartyForAdditionalOrderForm($prePayGWInfo, $_GET['subMethod']);
		} else {
			$orderToPayInfo = $this->getOrderToPayInfo($_GET['subMethod'], true);
			if (isset($_REQUEST['ccBrand'])) {
				$orderToPayInfo->ccBrand = $_REQUEST['ccBrand'];
			}

			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
				$this->procesReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
			}
			if ($redirectAction->inlineForm != null) {
				$this->assignSmartyForAdditionalOrderForm($prePayGWInfo, $_GET['subMethod'], $redirectAction->inlineForm);
			} else if ($redirectAction->redirectForm != null) {
				$this->uniModul->formRedirect($redirectAction->redirectForm);
			} else if ($redirectAction->redirectUrl != null) {
				header('Location: '.$redirectAction->redirectUrl);
				ResetUniErr();
				exit();
			}
		}
	}



	///// pomocne funkce

   // protoze presta 1.2.5 nema Tools::getHttpHost
   // volano z redirectu
   /** @return string  */
   public function getHttpHost() {
		return (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8');
   }

	/**
	 * @param mixed $cart
	 * @return bool
	 */
	private function _checkCurrency($cart)
	{
		$currency_order = new Currency((int)($cart->id_currency));
		$currencies_module = $this->paymentModule->getCurrency((int)$cart->id_currency);
		$currency_default = Configuration::get('PS_CURRENCY_DEFAULT');

		if (is_array($currencies_module))
			foreach ($currencies_module AS $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
		return false;
	}


	/// infoBox
	/**
	 * @param mixed $params
	 * @return ?string
	 */
	function hookLeftColumn($params)
	{
		if ($this->uniModul->name != 'GPWebPayGpe') {
			try {
				BeginUniErr();
				$infoBoxData = $this->uniModul->getInfoBoxData('PrestaShop', $this->getLanguage());
				$image = "UniModul/".$infoBoxData->image;

				global $smarty;
				$smarty->assign('ibtitle', $infoBoxData->title);
				$smarty->assign('iblink', $infoBoxData->link);
				$smarty->assign('ibimage', $image);
				$smarty->assign('platitiLink', $infoBoxData->platitiLink);
				$smarty->assign('platitiLinkText', $infoBoxData->platitiLinkText);
				return EndUniErr($this->display(__FILE__, 'infoBox.tpl'));
			} catch (Exception $ex) {
				ExceptionUniErr($ex);
			}
		}
		return null;
	}

	/**
	 * @param mixed $params
	 * @return ?string
	 */
	function hookRightColumn($params)
	{
		return $this->hookLeftColumn($params);
	}

	/**
	 * @param mixed $params
	 * @return ?string
	 */
	function hookFooter($params)
	{
		return $this->hookLeftColumn($params);
	}

	/**
	 * @param string $template
	 * @return string
	 */
	public function getTemplatePath($template) {
		if (Tools::file_exists_cache(_PS_THEME_DIR_.'modules/PrestaAdapter/'.basename($template))) {
			$template = _PS_THEME_DIR_.'modules/PrestaAdapter/'.basename($template);
		} else {
			$template = dirname(__FILE__).'/'.basename($template);
		}
		return $template;
	}


	/**
	 * @param string $template
	 * @return void
	 */
	public function displayTemplate($template) {
		if (_PS_VERSION_ >= '1.5.0') user_error("displayTemplate pouze pro PS <= 1.4");
		$tmpuri = $_SERVER['REQUEST_URI']; // hack kvuli body@id, ktere se odvozuje od url u modulu vcetne query stringu. body@id je potrebne pro stylovani a query string vadi
		$_SERVER['REQUEST_URI'] = preg_replace('/\?.*/', '', $_SERVER['REQUEST_URI']);
		include(dirname(__FILE__).'/../../header.php');
		$_SERVER['REQUEST_URI'] = $tmpuri;
		echo $this->display(dirname($template).'/PrestaAdapter', basename($template));
		include(dirname(__FILE__).'/../../footer.php');
	}

	/**
	 * @param string $callbackName
	 * @param mixed $arguments
	 * @return string
	 */
	function getCallbackUrl($callbackName, $arguments) {
		return $this->context->link->getModuleLink(strval($this->paymentModule->name),'Callback', ["_callbackName" => $callbackName] + $arguments);
	}

	/** @return string  */
	function getLanguage() {
		global $cookie;
		$lang = Tools::strtolower(Language::getIsoById(!empty($cookie->id_lang) ? (int)$cookie->id_lang : (int)Configuration::get('PS_LANG_DEFAULT')));
		if ($lang == 'gb') $lang = 'en'; // fix for mistakes in eshops
		return $lang;
	}


	/**
	 * @param PrePayGWInfo $prePayGWInfo
	 * @param string $subMethod
	 * @param ?string $inlineForm
	 * @return void
	 */
	function assignSmartyForAdditionalOrderForm($prePayGWInfo, $subMethod, $inlineForm = null) {

		global $smarty;
		$adapter = $this;

		$language = $adapter->getLanguage();
		$smarty->assign('iso_code', $language);
		$smarty->assign('uniModulLogo', $this->uniModul->getModulSubMethodLogoImage2($subMethod));
		$subname = $adapter->uniModul->getModuleSubMethodName($language, $subMethod);
		BeginUniErr(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
		$cart = isset($this->context->cart) ? $this->context->cart : $GLOBALS['cart'];  // phpStan: $this->context->cart may be nullable in older PS versions
		$smarty->assign('total', $cart->getOrderTotal(true, Cart::BOTH));
		$smarty->assign('totalf', Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH)));
		EndUniErr();
		$smarty->assign('methodName', $subname);
		$smarty->assign('minilogo', $adapter->uniModul->getMiniLogoSpan());
		$smarty->assign('ccBrand_title', $adapter->uniModul->dictionary->get('ccBrand_title', $language));

		if ($prePayGWInfo != null) {
			$smarty->assign('selectCsPayBrand', $prePayGWInfo->selectCsPayBrand);
			$smarty->assign('forexMessage', $prePayGWInfo->forexMessage);
		}

		$smarty->assign('inlineForm', $inlineForm);


		$smarty->assign('hide_left_column', true);
		$smarty->assign('hide_right_column', true);
		$smarty->assign('page_name', 'module-PrestaAdapter-redirect'); // nekdo chtel body id fini
	}

	/** @return void  */
	function assignSmartyForFailPage() {
		global $smarty;
		$adapter = $this;

		$smarty->assign('binderName', $adapter->name);
		$smarty->assign('errorMessage', $_REQUEST['errorMessage']);

		$smarty->assign('uniModulLogo', $this->uniModul->getModulSubMethodLogoImage2(""));

		$methodName = $adapter->uniModul->getModuleSubMethodName($adapter->getLanguage(), '');
		$smarty->assign('methodName', $methodName);

		$smarty->assign('hide_left_column', true);
		$smarty->assign('hide_right_column', true);
		$smarty->assign('minilogo', $adapter->uniModul->getMiniLogoSpan());
	}

}


class PrestaDbConnection implements IDatabaseConnection {

	/** @return void  */
	public function __construct() {
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
	}

	/**
	 * @param string $sql
	 * @return array<int, array<string, mixed>>
	 */
	public function sqlQuery($sql) {
		$res = Db::getInstance()->executeS($sql, true, false);
		if (!is_array($res)) throw new Exception("Query Failed! SQL: $sql - Error: ".Db::getInstance()->getMsgError());
		return $res;
	}

	/**
	 * @param string $sql
	 * @return void
	 */
	public function sqlExecute($sql) {
		$res = Db::getInstance()->execute($sql, false);
		if (!$res) throw new Exception("Query Failed! SQL: $sql - Error: ".Db::getInstance()->getMsgError());
	}

	/** @return int  */
	function getInsertId() {
		return (int)Db::getInstance()->Insert_ID();
	}

	/**
	 * @param ?scalar $text
	 * @return string
	 */
	function toSql($text) {
		if (is_null($text)) {
			return "null";
		} else {
			if (method_exists(Db::getInstance(), 'escape')) {
				return "'".Db::getInstance()->escape((string)$text, true, false)."'";
			} else {
				return "'".pSQL((string)$text)."'";  // PS <= 1.4
			}
		}
	}

}


if ("GPWebPayGpeNs" == "Pti"."UniNs") { // @phpstan-ignore-line
	class_alias(__NAMESPACE__ . '\\PrestaAdapter', 'PrestaAdapter');
}
